==> ginvite-2.3.4.tar.gz: ginvite/tests/src/Functional/GroupInviteTest.php <==
<?php

namespace Drupal\Tests\ginvite\Functional;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\ginvite\Plugin\GroupContentEnabler\GroupInvitation;
use Drupal\group\Entity\GroupContentInterface;
use Drupal\Tests\group\Functional\GroupBrowserTestBase;

/**
 * Tests the behavior of the group invite functionality.
 *
 * @group group
 */
class GroupInviteTest extends GroupBrowserTestBase {

  use StringTranslationTrait;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'group',
    'group_test_config',
    'ginvite',
  ];

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * The invitation manager.
   *
   * @var \Drupal\ginvite\GroupInvitationManager
   */
  protected $invitationManager;

  /**
   * The group we will use to test methods on.
   *
   * @var \Drupal\group\Entity\Group
   */
  protected $group;

  /**
   * The normal user we will use.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $account;

  /**
   * Gets the global (site) permissions for the group creator.
   *
   * @return string[]
   *   The permissions.
   */
  protected function getGlobalPermissions() {
    return [
      'view the administration theme',
      'access administration pages',
      'access group overview',
      'create default group',
      'create other group',
      'administer group',
      'bypass group access',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    $this->group = $this->createGroup(['uid' => $this->groupCreator->id()]);

    $this->account = $this->drupalCreateUser();
    $this->group->addMember($this->account);
    $this->group->save();

    $this->entityTypeManager = $this->container->get('entity_type.manager');
    $this->invitationManager = $this->container->get('ginvite.group_invitation_manager');
  }

  /**
   * Create invites and test general group invite behavior.
   */
  public function testInviteRolePermission() {
    $this->installPlugin();

    $this->drupalLogin($this->account);

    $role = $this->group->getGroupType()->getMemberRole();
    $role->grantPermissions(['invite users to group']);
    $role->save();

    // Verify the user cannot add roles to users on invite.
    $this->drupalGet("/group/{$this->group->id()}/content/add/group_invitation");
    $this->assertSession()->fieldNotExists('group_roles[default-custom]');

    // Add permissions to administer members to members of the group.
    $role = $this->group->getGroupType()->getMemberRole();
    $role->grantPermissions(['administer members']);
    $role->save();

    // Verify the normal member without the permission cannot add roles
    // to users on invite.
    $this->drupalGet("/group/{$this->group->id()}/content/add/group_invitation");
    $this->assertSession()->fieldExists('group_roles[default-custom]');
  }

  /**
   * We want to be sure user gets the group role.
   */
  public function testRoleAssigment() {
    $this->installPlugin();

    $account = $this->drupalCreateUser();

    $this->drupalLogin($account);

    $role_name = 'default-custom';
    $group_invitation = $this->invitationManager->createInvitation($this->group, $account->getEmail(), $account->id(), [$role_name]);
    $group_invitation->save();

    // Install and configure the Group Invitation plugin.
    $this->drupalGet("/ginvite/{$group_invitation->id()}/accept");
    $this->assertSession()->statusCodeEquals(200);

    $group_membership = $this->group->getMember($account);
    $this->assertTrue(in_array($role_name, array_keys($group_membership->getRoles())), 'Role has been found');
  }

  /**
   * Owner can access own invitations.
   */
  public function testAccessOwnInvitation() {
    $this->installPlugin();

    // Allow to see invitations.
    $role = $this->group->getGroupType()->getOutsiderRole();
    $role->grantPermissions(['view group invitations']);
    $role->save();

    $account = $this->drupalCreateUser();
    $this->drupalLogin($account);

    $group_invitation = $this->invitationManager->createInvitation($this->group, $account->getEmail(), $account->id());
    $group_invitation->save();

    $this->drupalGet("/ginvite/{$group_invitation->id()}/accept");
    $this->assertSession()->statusCodeEquals(200);

    $account = $this->drupalCreateUser();
    $this->drupalLogin($account);

    $group_invitation = $this->invitationManager->createInvitation($this->group, $account->getEmail(), $account->id());
    $group_invitation->save();

    $this->drupalGet("/ginvite/{$group_invitation->id()}/decline");
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Not owner can't access invitations.
   */
  public function testNotOwnerAccessRoutes() {
    $this->installPlugin();

    $account = $this->drupalCreateUser();

    $group_invitation = $this->invitationManager->createInvitation($this->group, $account->getEmail(), $account->id());
    $group_invitation->save();

    $not_owner_user = $this->drupalCreateUser();
    $this->drupalLogin($not_owner_user);

    // As not owner of invitation I can't accept or decline it.
    $this->checkRoutesAccess($group_invitation, 403);
  }

  /**
   * Access not pending invitation.
   */
  public function testAccessNotPendingInvitation() {
    $this->installPlugin();

    $account = $this->drupalCreateUser();

    $this->drupalLogin($account);

    // Allow to see invitations.
    $role = $this->group->getGroupType()->getOutsiderRole();
    $role->grantPermissions(['view group invitations']);
    $role->save();
  }

  /**
   * Check bulk operations routes access.
   */
  public function testBulkInvitationRoutes() {
    $this->installPlugin();

    $account = $this->drupalCreateUser();

    $this->drupalLogin($account);

    $this->drupalGet("/group/{$this->group->id()}/invite-members");
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet("/group/{$this->group->id()}/invite-members/confirm");
    $this->assertSession()->statusCodeEquals(403);

    // Allow to invite users.
    $role = $this->group->getGroupType()->getOutsiderRole();
    $role->grantPermissions(['invite users to group']);
    $role->save();

    $this->drupalGet("/group/{$this->group->id()}/invite-members");
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet("/group/{$this->group->id()}/invite-members/confirm");
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Check routes access.
   *
   * @param \Drupal\group\Entity\GroupContentInterface $group_invitation
   *   Group invitation.
   * @param int $status
   *   HTTP status.
   */
  private function checkRoutesAccess(GroupContentInterface $group_invitation, $status) {
    $this->drupalGet("/ginvite/{$group_invitation->id()}/accept");
    $this->assertSession()->statusCodeEquals($status);

    $this->drupalGet("/ginvite/{$group_invitation->id()}/decline");
    $this->assertSession()->statusCodeEquals($status);
  }

  /**
   * Install group invitation plugin.
   */
  private function installPlugin() {
    $this->drupalLogin($this->groupCreator);

    // Install and configure the Group Invitation plugin.
    $this->drupalGet('/admin/group/content/install/default/group_invitation');
    $this->submitForm([], 'Install plugin');
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('/admin/group/content/manage/default-group_invitation');
    $this->submitForm(['invitation_bypass_form' => 1], 'Save configuration');
    $this->assertSession()->statusCodeEquals(200);

    // Allow to see invitations.
    $role = $this->group->getGroupType()->getOutsiderRole();
    $role->grantPermissions(['view group invitations']);
    $role->save();

    drupal_flush_all_caches();
  }

}

==> ginvite-3.0.0-rc2.tar.gz: ginvite/tests/src/Functional/GroupInviteTest.php <==
<?php

namespace Drupal\Tests\ginvite\Functional;

use Drupal\Core\StringTranslation\StringTranslationTrait;
use Drupal\ginvite\Plugin\Group\Relation\GroupInvitation;
use Drupal\group\Entity\GroupRelationshipInterface;
use Drupal\group\PermissionScopeInterface;
use Drupal\Tests\group\Functional\GroupBrowserTestBase;
use Drupal\user\RoleInterface;

/**
 * Tests the behavior of the group invite functionality.
 *
 * @group group
 */
class GroupInviteTest extends GroupBrowserTestBase {

  use StringTranslationTrait;

  /**
   * The invitation manager.
   *
   * @var \Drupal\ginvite\GroupInvitationManager
   */
  protected $invitationManager;

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'group',
    'group_test_config',
    'ginvite',
  ];

  /**
   * The group we will use to test methods on.
   *
   * @var \Drupal\group\Entity\Group
   */
  protected $group;

  /**
   * The normal user we will use.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $account;

  /**
   * The normal user we will use.
   *
   * @var \Drupal\group\Entity\GroupRoleInterface
   */
  protected $groupIndividualRole;

  /**
   * The outsider role.
   *
   * @var \Drupal\group\Entity\GroupRoleInterface
   */
  protected $groupOutsiderRole;

  /**
   * The member role.
   *
   * @var \Drupal\group\Entity\GroupRoleInterface
   */
  protected $groupMemberRole;

  /**
   * Gets the global (site) permissions for the group creator.
   *
   * @return string[]
   *   The permissions.
   */
  protected function getGlobalPermissions() {
    return [
      'view the administration theme',
      'access administration pages',
      'access group overview',
      'create default group',
      'administer group',
    ];
  }

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->setUpAccount();

    $this->group = $this->createGroup([
      'uid' => $this->groupCreator->id(),
      'type' => 'default',
    ]);

    $this->invitationManager = $this->container->get('ginvite.group_invitation_manager');

    $this->account = $this->drupalCreateUser();
    $this->group->addMember($this->account);
    $this->group->save();

    $this->groupIndividualRole = $this->createGroupRole([
      'group_type' => $this->group->getGroupType()->id(),
      'scope' => PermissionScopeInterface::INDIVIDUAL_ID,
      'permissions' => ['view group'],
    ]);

    $this->installPlugin();
  }

  /**
   * Install group invitation plugin.
   */
  private function installPlugin() {
    $this->drupalLogin($this->groupCreator);

    // Install and configure the Group Invitation plugin.
    $this->drupalGet('/admin/group/content/install/default/group_invitation');
    $this->submitForm([], 'Install plugin');
    $this->assertSession()->statusCodeEquals(200);

    $this->drupalGet('/admin/group/content/manage/default-group_invitation');
    $this->submitForm(['invitation_bypass_form' => 1], 'Save configuration');
    $this->assertSession()->statusCodeEquals(200);

    // Add permissions to invite users to members of the group.
    $this->groupOutsiderRole = $this->createGroupRole([
      'group_type' => $this->group->getGroupType()->id(),
      'scope' => PermissionScopeInterface::OUTSIDER_ID,
      'global_role' => RoleInterface::AUTHENTICATED_ID,
      'permissions' => [
        'view group invitations',
        'view group',
      ],
    ]);

    $this->groupMemberRole = $this->createGroupRole([
      'group_type' => $this->group->getGroupType()->id(),
      'scope' => PermissionScopeInterface::INSIDER_ID,
      'global_role' => RoleInterface::AUTHENTICATED_ID,
      'permissions' => [
        'view group invitations',
        'view group',
      ],
    ]);

    drupal_flush_all_caches();
  }

  /**
   * Check routes access.
   *
   * @param \Drupal\group\Entity\GroupRelationshipInterface $group_invitation
   *   Group invitation.
   * @param int $status
   *   HTTP status.
   */
  private function checkRoutesAccess(GroupRelationshipInterface $group_invitation, $status) {
    $this->drupalGet("/ginvite/{$group_invitation->id()}/accept");
    $this->assertSession()->statusCodeEquals($status);

    $this->drupalGet("/ginvite/{$group_invitation->id()}/decline");
    $this->assertSession()->statusCodeEquals($status);
  }

  /**
   * Create invites and test general group invite behavior.
   */
  public function testInviteRolePermission() {

    $this->drupalLogin($this->account);

    $this->groupMemberRole->grantPermissions(['invite users to group']);
    $this->groupMemberRole->save();

    $this->drupalGet("/group/{$this->group->id()}/content/add/group_invitation");
    $this->assertSession()->fieldNotExists("group_roles[{$this->groupIndividualRole->id()}]");

    $this->groupMemberRole->grantPermissions(['administer members']);
    $this->groupMemberRole->save();

    $this->drupalGet("/group/{$this->group->id()}/content/add/group_invitation");
    $this->assertSession()->fieldExists("group_roles[{$this->groupIndividualRole->id()}]");

    $this->groupMemberRole->revokePermissions(['administer members', 'invite users to group']);
    $this->groupMemberRole->save();
  }

  /**
   * We want to be sure user gets the group role.
   */
  public function testRoleAssigment() {

    $account = $this->drupalCreateUser();

    $this->drupalLogin($account);

    $group_invitation = $this->invitationManager->createInvitation($this->group, $account->getEmail(), $account->id(), [$this->groupIndividualRole->id()]);
    $group_invitation->save();

    // Install and configure the Group Invitation plugin.
    $this->drupalGet("/ginvite/{$group_invitation->id()}/accept");
    $this->assertSession()->statusCodeEquals(200);

    $group_membership = $this->group->getMember($account);
    $this->assertTrue(in_array($this->groupIndividualRole->id(), array_keys($group_membership->getRoles())), 'Role has been found');
  }

  /**
   * Owner can access own invitations.
   */
  public function testAccessOwnInvitation() {
    $account = $this->drupalCreateUser();
    $this->drupalLogin($account);

    $group_invitation = $this->invitationManager->createInvitation($this->group, $account->getEmail(), $account->id());
    $group_invitation->save();

    $this->drupalGet("/ginvite/{$group_invitation->id()}/accept");
    $this->assertSession()->statusCodeEquals(200);

    // We need to create another invitation, because the previous is accepted.
    $account = $this->drupalCreateUser();
    $this->drupalLogin($account);

    $group_invitation = $this->invitationManager->createInvitation($this->group, $account->getEmail(), $account->id());
    $group_invitation->save();

    $this->drupalGet("/ginvite/{$group_invitation->id()}/decline");
    $this->assertSession()->statusCodeEquals(200);
  }

  /**
   * Not owner can't access invitations.
   */
  public function testNotOwnerAccessRoutes() {

    $account = $this->drupalCreateUser();

    $group_invitation = $this->invitationManager->createInvitation($this->group, $account->getEmail(), $account->id());
    $group_invitation->save();

    $not_owner_user = $this->drupalCreateUser();
    $this->drupalLogin($not_owner_user);

    // As not owner of invitation I can't accept or decline it.
    $this->checkRoutesAccess($group_invitation, 403);
  }

  /**
   * Access not pending invitation.
   */
  public function testAccessNotPendingInvitation() {

    $group_invitation = $this->invitationManager->createInvitation($this->group, $this->account->getEmail(), $this->account->id());
    $group_invitation->save();

    $account = $this->drupalCreateUser();

    $this->drupalLogin($account);

    // As not owner of invitation I can't accept or decline it.
    $this->checkRoutesAccess($group_invitation, 403);

    $group_invitation = $this->invitationManager->createInvitation($this->group, $this->account->getEmail(), $this->account->id());
    $group_invitation->save();
    $group_invitation->set('invitation_status', GroupInvitation::INVITATION_ACCEPTED)->save();

    $this->checkRoutesAccess($group_invitation, 403);
  }

  /**
   * Check bulk operations routes access.
   */
  public function testBulkInvitationRoutes() {
    $account = $this->drupalCreateUser();

    $this->drupalLogin($account);

    $this->groupOutsiderRole->grantPermission('invite users to group');
    $this->groupOutsiderRole->save();

    $this->drupalGet("/group/{$this->group->id()}/invite-members");
    $this->assertSession()->statusCodeEquals(200);
    $this->drupalGet("/group/{$this->group->id()}/invite-members/confirm");
    $this->assertSession()->statusCodeEquals(200);

    $this->groupOutsiderRole->revokePermission('invite users to group');
    $this->groupOutsiderRole->save();

    $this->drupalGet("/group/{$this->group->id()}/invite-members");
    $this->assertSession()->statusCodeEquals(403);
    $this->drupalGet("/group/{$this->group->id()}/invite-members/confirm");
    $this->assertSession()->statusCodeEquals(403);
  }

}
